PS_BASE   = 0xFFFF4000      /* Power saving controller. */
AIC_BASE  = 0xFFFFF000      /* Advanced interrupt controller. */
AIC_SVR   = 0x080           /* Source Vector Register */
AIC_EOICR = 0x130           /*  of Interrupt Command Register */
AIC_SPU   = 0x134           /* Spurious Vector Register */

FLASH_BASE = 0x01000000
FLASH_SIZE = 0x00200000

RAM_BASE  = 0x00000000
RAM_LIMIT = 0x00000100
RAM_SIZE  = (256*1024)

IRQ_STACK_SIZE = (3*8*4)     @ 3 words per interrupt priority level
FIQ_STACK_SIZE = (3*4)       @ 3 words
ABT_STACK_SIZE = (1*4)       @ 1 word
UND_STACK_SIZE = (1*4)       @ 1 word

ARM_MODE_USER = 0x10
ARM_MODE_FIQ = 0x11
ARM_MODE_IRQ = 0x12
ARM_MODE_SVC = 0x13
ARM_MODE_ABORT = 0x17
ARM_MODE_UNDEF = 0x1B
ARM_MODE_SYS = 0x1F


        .globl  __main
        .globl  undefvec
        .globl  swivec
__main:
        b       InitReset
@undefvec:
@        b       undefvec
@swivec:
@        b       swivec
@pabtvec:
@        b       pabtvec
@dabtvec:
@        b       dabtvec
@rsvdvec:
@        b       rsvdvec
@irqvec:
@        b       irqvec
@fiqvec:
@        b       fiqvec

/*
 * This will be copied to address 0.
 */
        .globl  VectorTable
VectorTable:
        ldr     pc, [pc, #0x18]     /* SoftReset */
        ldr     pc, [pc, #0x18]     /* UndefHandler */
        ldr     pc, [pc, #0x18]     /* SWIHandler */
        ldr     pc, [pc, #0x18]     /* PrefetchAbortHandler */
        ldr     pc, [pc, #0x18]     /* DataAbortHandler */
        nop                         /* Unused */
        ldr     pc, [pc, #-0xF20]   /* Interrupt request, auto vectoring. */
        ldr     pc, [pc, #-0xF20]   /* Fast interrupt request, auto vectoring. */

        .globl  VectorOffsets
VectorOffsets:
        .word   SoftReset
        .word   UndefHandler
        .word   SWIHandler
        .word   PrefetchAbortHandler
        .word   DataAbortHandler


/*
 * Default handlers.
 */
SoftReset:
        b       SoftReset
UndefHandler:
        b       UndefHandler
        .globl SWIHandler
SWIHandler:
        b       SWIHandler  /* May be used for semi hosting */
PrefetchAbortHandler:
        b       PrefetchAbortHandler
DataAbortHandler:
        b       DataAbortHandler




        .globl	InitReset
InitReset:
        /*
         * Enable all clocks.
         */
        mvn     r0, #0
        ldr     r1, =PS_BASE
        str     r0, [r1, #0x04]

        /*
         * Set interrupt controller.
         */
        add     r0, pc,#-(8+.-AicData)
        ldmia   r0, {r1-r4}

@- Setup the Spurious Vector
        str     r4, [r1, #AIC_SPU]      @ r4 = spurious handler


@- ICE note : For ICE debug 
@- Perform 8 End Of Interrupt Command to make sure AIC will not lock out nIRQ
        mov     r0, #8
LoopAic0:
        str     r1, [r1, #AIC_EOICR]    @ any value written
        subs    r0, r0, #1
        bhi     LoopAic0

@- Set up the default interrupt handler vectors
        str     r2, [r1, #AIC_SVR]      @ SVR[0] for FIQ
        add     r1, r1, #AIC_SVR
        mov     r0, #31                 @ counter
LoopAic1:
        str     r3, [r1, r0, LSL #2]    @ SVRs for IRQs
        subs    r0, r0, #1              @ do not save FIQ
        bhi     LoopAic1

        b       EndInitAic

        .globl  AicData
AicData:
        .word   AIC_BASE                @ AIC Base Address

        .globl  PtDefaultHandler
PtDefaultHandler:
        .word   at91_default_fiq_handler
        .word   at91_default_irq_handler
        .word   at91_spurious_handler

        /*
         * Copy vectors to RAM address 0.
         */
EndInitAic:
        mov     r8, #0
        add     r9, pc, #-(8+.-VectorTable)
        ldmia   r9!, {r0-r7}
        stmia   r8!, {r0-r7}
        ldmia   r9!, {r0-r4}
        stmia   r8!, {r0-r4}

        /*
         * Set exception stack pointers.
         */
        ldr     r0, =0x100
        msr     CPSR_c, #ARM_MODE_FIQ | 0xC0
        mov     r13, r0
        sub     r0, r0, #FIQ_STACK_SIZE
        msr     CPSR_c, #ARM_MODE_IRQ | 0xC0
        mov     r13, r0
        sub     r0, r0, #IRQ_STACK_SIZE
        msr     CPSR_c, #ARM_MODE_ABORT | 0xC0
        mov     r13, r0
        sub     r0, r0, #ABT_STACK_SIZE
        msr     CPSR_c, #ARM_MODE_UNDEF | 0xC0
        mov     r13, r0
        sub     r0, r0, #UND_STACK_SIZE
        msr     CPSR_c, #ARM_MODE_SVC | 0xC0
        mov     r13, r0

        /*
         * Set user mode.
         */
        msr     CPSR_c, #ARM_MODE_USER

        /*
         * Clear bss.
         */
        ldr     r1, =__bss_start__
        ldr     r2, =__bss_end__
        ldr     r3, =0
        .globl  clear_bss
clear_bss:
        cmp     r1, r2
        strne   r3, [r1], #+4
        bne     clear_bss

        /*
         * Initialize user stack pointer.
         */
        ldr     r13, =_stack_top

        /*
         * Jump to Nut/OS initialization.
         */
        ldr     r0, =main
        bx      r0

End:
        b       End



        .globl  __bss_top
__bss_top:  
        .long   __bss_top__
