/*
 *  head.S
 *
 *  Copyright (C) 2001-2002 Hayden Luo
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *  32-bit startup code for s3c5410b board
 */

#include <dev/s3c4510b_hw.h>

        .extern ISR_IrqHandler
        .extern NutInit
        .extern NutHWInit
        .extern INT_EXCEPT
        .extern INT_INVALID_CODE

        .globl  TheEntry

		.globl  GetCPSR

        .section ".init"

        @ Address 0x8000 SDRAM

TheEntry:
        b       __init_entry


        .section ".text"
__init_entry:
/*
 * Setup Register SYSCFG
 *    - Special Register Base Addr : 0x3ff0000
 *    - Cache and Write Buffer Enable according to the environments
 *    - Check DRAM type : Normal / SDRAM
 */
        ldr     r0, =SYSCFG

        /* Start_addr = 0x3FF00000 */
        ldr     r1, =0x87FFFF80

        /* Cache OFF, Write Buffer OFF, 4K SRAM, 4K CACHE */
        str     r1, [r0]

/*
 * SysInitVars: Initialise the DATA and BSS sections.
 *
 * The DATA section is initialised by copying data from the end of the
 * ROM image (given by _etext) to the start of the RAM image
 * (given by __data_start), stopping when we reach _edata.
 * All data from _edata to _end is then cleared to 0
 */

/*
 * Load up the linker defined values for the static data copy
 */
        ldr     r0, =_etext
        ldr     r1, =_data_start
        ldr     r3, =_edata

/*
 * But first check whether we are trying to copy to the same address.
 * If so, this means that the image was linked as an application image
 * with the DATA section immediately following the CODE section.
 * Therefore there is nothing to copy since the data is already in place
 */
        cmp     r0, r1
        beq     1f

/*
 * Stop on CS (ie R1 becomes >= R3).
 */
0:      cmp     r1, r3
        ldrcc   r2, [r0], #4
        strcc   r2, [r1], #4
        bcc     0b

/*
 * Clear remainder of data to _end to 0
 */
1:      ldr     r1, =__bss_end
        mov     r2, #0
2:      cmp     r3, r1
        strcc   r2, [r3], #4
        bcc     2b

/*
 * Set up vector
 */
        mov     r8,     #0
        adr     r9,     VECTABLE
        ldmia   r9!,    {r0-r7}
        stmia   r8!,    {r0-r7}
        ldmia   r9!,    {r0-r7}
        stmia   r8!,    {r0-r7}

/*
 * Now set up stack
 */
        ldr     r8,   =__stack

        mrs     r1,   cpsr
        bic     r1,   r1, #PSR_MODE_MASK
        orr     r1,   r1, #0xC0

        orr     r2,   r1, #PSR_FIQ_MODE
        orr     r3,   r1, #PSR_SVC_MODE
        msr     cpsr, r2
        mov     sp,   r8
        sub     r8,   r8, #FIQ_STACK_SIZE

        orr     r2,   r1, #PSR_IRQ_MODE
        orr     r3,   r1, #PSR_SVC_MODE
        msr     cpsr, r2
        mov     sp,   r8
        sub     r8,   r8, #IRQ_STACK_SIZE

        orr     r2,   r1, #PSR_ABORT_MODE
        orr     r3,   r1, #PSR_SVC_MODE
        msr     cpsr, r2
        mov     sp,   r8
        sub     r8,   r8, #ABT_STACK_SIZE

        orr     r2,   r1, #PSR_UNDEF_MODE
        orr     r3,   r1, #PSR_SVC_MODE
        msr     cpsr, r2
        mov     sp,   r8
        sub     r8,   r8, #UND_STACK_SIZE

        orr     r2,   r1, #PSR_SVC_MODE
/*
 * 	enable irq
 */
        bic     r2,   r2, #0x80
        msr     cpsr, r2
        mov     sp,   r8

/*
 * Cache Flush & Enable
 */
        ldr     r0, =SYSCFG
        ldr     r1, =0x87FFFF90         /* Start_addr = 0x3FF00000 */

        /* Cache OFF, Write Buffer OFF, 0K SRAM, 8K CACHE */
        str     r1, [r0]

        ldr     r0, =0x11000000         /* Cache Flush */
        mov     r1, #0
        mov     r2, #256
1:
        str     r1, [r0], #4
        subs    r2, r2, #1
        bne     1b

        nop
        ldr     r0, =SYSCFG
        ldr     r2, [r0]
        orr     r2, r2, #6
        str     r2, [r0]                /* Cache, Write Buffer Enable */

/*
 * Now go to Nut/OS
 */
        b       NutInit
        /* never return */

/*
 * Get CPSR
 */
GetCPSR:
        mrs     r0, cpsr
        mov     pc, lr

/*
 * Int handlers
 */
        .align  5
AIRQ_Handle:
        stmfd   sp!, {r0-r12, lr}
        bl      ISR_IrqHandler
        ldmfd   sp!, {r0-r12, lr}
        subs    pc,  lr, #4

AINT_EXCEPT:
        b       INT_EXCEPT
        /* never return */
        /* FIXME */

AINT_INVALID_CODE:
		mov     r0,  lr
		b       INT_INVALID_CODE
        /* never return */
		/* FIXME */
/*
 * vector table
 */

VECTABLE:
        ldr       pc, AddressOfReset
        ldr       pc, AddressOfUndefined
        ldr       pc, AddressOfSWI
        ldr       pc, AddressOfPrefetch
        ldr       pc, AddressOfAbort
        nop
        ldr       pc, AddressOfIRQ
        ldr       pc, AddressOfFIQ

AddressOfReset     : .word TheEntry
AddressOfUndefined : .word AINT_INVALID_CODE
AddressOfSWI       : .word AINT_EXCEPT
AddressOfPrefetch  : .word AINT_EXCEPT
AddressOfAbort     : .word AINT_EXCEPT
                     .word 0
AddressOfIRQ       : .word AIRQ_Handle
AddressOfFIQ       : .word AINT_EXCEPT


/*
 * Define stack for supervisor modes
 */

        .align
        .section ".bss"
